/**
 * browser ios 유무 반환
 * @memberof module:browser
 * @example
 * import { isAOS } from 'fe-code-snippet/browser';
 * console.log(isAOS);
 * @returns boolean
 */
const isAOS = () => {
    return /Android/.test(window.navigator.userAgent);
};

// @see https://davidwalsh.name/vendor-prefix
const prefix = (() => {
    const styles = window.getComputedStyle(window.document.documentElement);
    const pre = (Array.from(styles).join('').match(/-(moz|webkit|ms)-/) || (styles.OLink === '' && ['', 'o']))[1];
    const dom = ('Webkit|Moz|MS|O').match(new RegExp(`(${pre})`, 'i'))[1];
    return {
        dom,
        lowercase: pre,
        css: `-${pre}-`,
        js: pre[0].toUpperCase() + pre.substr(1)
    };
})();

/*
 * Public API Surface of @nonoll/code-snippet/browser
 */

var publicApi_browser = /*#__PURE__*/Object.freeze({
    __proto__: null,
    isAOS: isAOS,
    prefix: prefix
});

function noop() { }

class EventEmitter {
    constructor() {
        this.events = [];
    }
    on(eventName, listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    off(eventName, listener) {
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        if (!~matched) {
            return;
        }
        this.events.splice(matched, 1);
        return this;
    }
    once(eventName, listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    fire(eventName, ...values) {
        this.emit(eventName, values);
        return this;
    }
}

/*
 * Public API Surface of @nonoll/code-snippet/event
 */

var publicApi_event = /*#__PURE__*/Object.freeze({
    __proto__: null,
    EventEmitter: EventEmitter
});

const debounce = (fn, delay) => {
    let timer;
    delay = delay || 0;
    return (...args) => {
        window.clearTimeout(timer);
        timer = window.setTimeout(() => {
            fn.apply(null, args);
        }, delay);
    };
};

const throttle = (fn, delay) => {
    delay = delay || 0;
    const caller = (args) => {
        fn.apply(null, args);
        base = +new Date();
    };
    let base = null;
    let isLeading = true;
    let stamp = null;
    return (...args) => {
        if (isLeading) {
            caller(args);
            isLeading = false;
            return;
        }
        stamp = +new Date();
        base = base || stamp;
        if ((stamp - base) >= delay) {
            caller(args);
        }
    };
};

/*
 * Public API Surface of @nonoll/code-snippet/functions
 */

var publicApi_functions = /*#__PURE__*/Object.freeze({
    __proto__: null,
    debounce: debounce,
    noop: noop,
    throttle: throttle
});

/**
 * MutationRecordType
 * @export
 * @readonly
 * @enum {string}
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @export
 * @readonly
 * @enum {string}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    /** change childList */
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    /** change subtree */
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    /** change attributes */
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    /** change characterData */
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    /** change other case */
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    /** change wildcard - all case */
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
class PureMutationObserver {
    constructor(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    observe(target, options) {
        this.pure.observe(target, options);
    }
    disconnect() {
        this.pure.disconnect();
    }
}
/**
 * @export
 * @class MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @example
 * import { MutationObserver, MUTATION_EVENTS } from '@nonoll/my-lib/observer';
 *
 * const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
 *  const doc = window.document;
 *  const target = doc.createElement(tag);
 *  target.setAttribute('id', id);
 *  target.setAttribute('style', style);
 *  target.setAttribute('value', value);
 *  if (text) {
 *    target.textContent = text;
 *  }
 *  return target;
 * }
 *
 * const forExample = () => {
 *  const doc = window.document;
 *
 *  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
 *  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
 *  const appendButton = createElement({ tag: 'button', text: 'append' });
 *
 *  doc.body.appendChild(attachButton);
 *  doc.body.appendChild(detachButton);
 *  doc.body.appendChild(appendButton);
 *
 *  attachButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('attachButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    observer.attach();
 *  });
 *
 *  detachButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('detachButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    observer.detach();
 *  });
 *
 *  appendButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('appendButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    const input = createElement({ tag: 'input', value: `${+new Date()}` });
 *    target.appendChild(input);
 *  });
 * }
 *
 * const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
 * window.document.body.appendChild(target);
 *
 * const options = {
 *  childList: true,
 *  subtree: true
 * };
 *
 * const observer = new MutationObserver({ target, options });
 * observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
 *  console.log('wildCard', type, values);
 * }).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
 *  console.log('childList', values);
 * });
 * observer.attach();
 *
 * forExample();
 */
class MutationObserver extends EventEmitter {
    constructor({ target = null, debounce: debounce$1 = 300, callback = noop, options = {} }) {
        super();
        const defaultOptions = { childList: true, subtree: true };
        this.target = target;
        this.options = Object.assign({}, defaultOptions, options);
        this.onMutationObserverListener = this.onMutationObserverListener.bind(this);
        this.callback = callback || noop;
        if (debounce$1) {
            this.observer = new PureMutationObserver(debounce(this.onMutationObserverListener, debounce$1));
        }
        else {
            this.observer = new PureMutationObserver(this.onMutationObserverListener);
        }
    }
    attach() {
        this.observer.observe(this.target, this.options);
        return this;
    }
    onMutationObserverListener(mutations) {
        mutations.forEach(mutation => {
            const type = mutation.type;
            const values = { type, mutation };
            switch (type) {
                case MutationRecordType.childList:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.subtree:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.attributes:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.characterData:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                default:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    }
    detach() {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    }
    destroy() {
        this.detach();
        this.observer = null;
        return this;
    }
}

const doc = window.document;
function getVendorPrefix() {
    let prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    }
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    }
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
const VENDOR_PREFIX = getVendorPrefix();
const VISIBILITY_EVENT_NAME = `${VENDOR_PREFIX}visibilitychange`;
const HIDDEN_METHOD_NAME = VENDOR_PREFIX ? `${VENDOR_PREFIX}Hidden` : 'hidden';
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
class VisibilityChangeObserver extends EventEmitter {
    constructor() {
        super();
        if (!VisibilityChangeObserver.isSupportVisibilityChange()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        this.onVisibilityChangeListener = this.onVisibilityChangeListener.bind(this);
    }
    static visibilityEventName() {
        return `${VENDOR_PREFIX}visibilitychange`;
    }
    static hiddenMethodName() {
        return VENDOR_PREFIX ? `${VENDOR_PREFIX}Hidden` : 'hidden';
    }
    static isSupportVisibilityChange() {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    }
    onVisibilityChangeListener() {
        const isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden, isShow: !isHidden });
    }
    isHidden() {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    }
    getStatus() {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        return this;
    }
    attach() {
        if (!doc.addEventListener) {
            return;
        }
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    detach() {
        if (!doc.addEventListener) {
            return;
        }
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    destroy() {
        this.detach();
        return this;
    }
}

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

var publicApi_observer = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get MutationRecordType () { return MutationRecordType; },
    get MUTATION_EVENTS () { return MUTATION_EVENTS; },
    MutationObserver: MutationObserver,
    getVendorPrefix: getVendorPrefix,
    VENDOR_PREFIX: VENDOR_PREFIX,
    VISIBILITY_EVENT_NAME: VISIBILITY_EVENT_NAME,
    HIDDEN_METHOD_NAME: HIDDEN_METHOD_NAME,
    get VISIBILITY_EVENTS () { return VISIBILITY_EVENTS; },
    VisibilityChangeObserver: VisibilityChangeObserver
});

/*
 * Public API Surface of @nonoll/code-snippet/types
 */

var publicApi_types = /*#__PURE__*/Object.freeze({
    __proto__: null
});

/*
 * Public API Surface of @nonoll/code-snippet
 */

/**
 * Generated bundle index. Do not edit.
 */

export { publicApi_browser as browser, publicApi_event as event, publicApi_functions as functions, publicApi_observer as observer, publicApi_types as types };
//# sourceMappingURL=nonoll-code-snippet.js.map
