import { __extends } from 'tslib';

function noop() { }

var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this.events = [];
    }
    EventEmitter.prototype.on = function (eventName, listener, context) {
        if (listener === void 0) { listener = noop; }
        this.events.push({ eventName: eventName, listener: listener, context: context });
        return this;
    };
    EventEmitter.prototype.off = function (eventName, listener) {
        var matched = this.events.findIndex(function (regEvent) { return regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true); });
        if (!~matched) {
            return;
        }
        this.events.splice(matched, 1);
        return this;
    };
    EventEmitter.prototype.once = function (eventName, listener, context) {
        var _this = this;
        if (listener === void 0) { listener = noop; }
        var onceWrapper = function () {
            var values = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                values[_i] = arguments[_i];
            }
            _this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    };
    EventEmitter.prototype.emit = function (eventName) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        this.events.forEach(function (regEvent) {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    };
    EventEmitter.prototype.fire = function (eventName) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        this.emit(eventName, values);
        return this;
    };
    return EventEmitter;
}());

var debounce = function (fn, delay) {
    var timer;
    delay = delay || 0;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        window.clearTimeout(timer);
        timer = window.setTimeout(function () {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * MutationRecordType
 * @export
 * @readonly
 * @enum {string}
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @export
 * @readonly
 * @enum {string}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    /** change childList */
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    /** change subtree */
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    /** change attributes */
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    /** change characterData */
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    /** change other case */
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    /** change wildcard - all case */
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
var PureMutationObserver = /** @class */ (function () {
    function PureMutationObserver(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    PureMutationObserver.prototype.observe = function (target, options) {
        this.pure.observe(target, options);
    };
    PureMutationObserver.prototype.disconnect = function () {
        this.pure.disconnect();
    };
    return PureMutationObserver;
}());
/**
 * @export
 * @class MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @example
 * import { MutationObserver, MUTATION_EVENTS } from '@nonoll/my-lib/observer';
 *
 * const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
 *  const doc = window.document;
 *  const target = doc.createElement(tag);
 *  target.setAttribute('id', id);
 *  target.setAttribute('style', style);
 *  target.setAttribute('value', value);
 *  if (text) {
 *    target.textContent = text;
 *  }
 *  return target;
 * }
 *
 * const forExample = () => {
 *  const doc = window.document;
 *
 *  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
 *  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
 *  const appendButton = createElement({ tag: 'button', text: 'append' });
 *
 *  doc.body.appendChild(attachButton);
 *  doc.body.appendChild(detachButton);
 *  doc.body.appendChild(appendButton);
 *
 *  attachButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('attachButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    observer.attach();
 *  });
 *
 *  detachButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('detachButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    observer.detach();
 *  });
 *
 *  appendButton.addEventListener('click', e => {
 *    e.preventDefault();
 *    console.log('appendButton clicked');
 *    if (!observer) {
 *      return;
 *    }
 *    const input = createElement({ tag: 'input', value: `${+new Date()}` });
 *    target.appendChild(input);
 *  });
 * }
 *
 * const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
 * window.document.body.appendChild(target);
 *
 * const options = {
 *  childList: true,
 *  subtree: true
 * };
 *
 * const observer = new MutationObserver({ target, options });
 * observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
 *  console.log('wildCard', type, values);
 * }).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
 *  console.log('childList', values);
 * });
 * observer.attach();
 *
 * forExample();
 */
var MutationObserver = /** @class */ (function (_super) {
    __extends(MutationObserver, _super);
    function MutationObserver(_a) {
        var _b = _a.target, target = _b === void 0 ? null : _b, _c = _a.debounce, debounce$1 = _c === void 0 ? 300 : _c, _d = _a.callback, callback = _d === void 0 ? noop : _d, _e = _a.options, options = _e === void 0 ? {} : _e;
        var _this = _super.call(this) || this;
        var defaultOptions = { childList: true, subtree: true };
        _this.target = target;
        _this.options = Object.assign({}, defaultOptions, options);
        _this.onMutationObserverListener = _this.onMutationObserverListener.bind(_this);
        _this.callback = callback || noop;
        if (debounce$1) {
            _this.observer = new PureMutationObserver(debounce(_this.onMutationObserverListener, debounce$1));
        }
        else {
            _this.observer = new PureMutationObserver(_this.onMutationObserverListener);
        }
        return _this;
    }
    MutationObserver.prototype.attach = function () {
        this.observer.observe(this.target, this.options);
        return this;
    };
    MutationObserver.prototype.onMutationObserverListener = function (mutations) {
        var _this = this;
        mutations.forEach(function (mutation) {
            var type = mutation.type;
            var values = { type: type, mutation: mutation };
            switch (type) {
                case MutationRecordType.childList:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.subtree:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.attributes:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                case MutationRecordType.characterData:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
                default:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    };
    MutationObserver.prototype.detach = function () {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    };
    MutationObserver.prototype.destroy = function () {
        this.detach();
        this.observer = null;
        return this;
    };
    return MutationObserver;
}(EventEmitter));

var doc = window.document;
function getVendorPrefix() {
    var prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    }
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    }
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
var VENDOR_PREFIX = getVendorPrefix();
var VISIBILITY_EVENT_NAME = VENDOR_PREFIX + "visibilitychange";
var HIDDEN_METHOD_NAME = VENDOR_PREFIX ? VENDOR_PREFIX + "Hidden" : 'hidden';
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
var VisibilityChangeObserver = /** @class */ (function (_super) {
    __extends(VisibilityChangeObserver, _super);
    function VisibilityChangeObserver() {
        var _this = _super.call(this) || this;
        if (!VisibilityChangeObserver.isSupportVisibilityChange()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        _this.onVisibilityChangeListener = _this.onVisibilityChangeListener.bind(_this);
        return _this;
    }
    VisibilityChangeObserver.visibilityEventName = function () {
        return VENDOR_PREFIX + "visibilitychange";
    };
    VisibilityChangeObserver.hiddenMethodName = function () {
        return VENDOR_PREFIX ? VENDOR_PREFIX + "Hidden" : 'hidden';
    };
    VisibilityChangeObserver.isSupportVisibilityChange = function () {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    };
    VisibilityChangeObserver.prototype.onVisibilityChangeListener = function () {
        var isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden: isHidden, isShow: !isHidden });
    };
    VisibilityChangeObserver.prototype.isHidden = function () {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    };
    VisibilityChangeObserver.prototype.getStatus = function () {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        return this;
    };
    VisibilityChangeObserver.prototype.attach = function () {
        if (!doc.addEventListener) {
            return;
        }
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    };
    VisibilityChangeObserver.prototype.detach = function () {
        if (!doc.addEventListener) {
            return;
        }
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    };
    VisibilityChangeObserver.prototype.destroy = function () {
        this.detach();
        return this;
    };
    return VisibilityChangeObserver;
}(EventEmitter));

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HIDDEN_METHOD_NAME, MUTATION_EVENTS, MutationObserver, MutationRecordType, VENDOR_PREFIX, VISIBILITY_EVENTS, VISIBILITY_EVENT_NAME, VisibilityChangeObserver, getVendorPrefix };
//# sourceMappingURL=nonoll-code-snippet-observer.js.map
